SupportedMinimumAlignment

Trait SupportedMinimumAlignment 

pub trait SupportedMinimumAlignment: Sealed {
    const VALUE: usize;
}
Statically guarantees that a minimum alignment is supported.

This trait is sealed: the list of implementors below is total. Users do not have the ability to mark additional MinimumAlignment<N> values as supported. Only bump allocators with the supported minimum alignments are constructable.

Required Associated Constants

const VALUE: usize

The minimum alignment in bytes.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors