bump_scope

Module stats

Types that provide statistics about the memory usage of the bump allocator.

Structs

  • Refers to a chunk of memory that was allocated by the bump allocator.
  • Iterator that iterates over next chunks by continuously calling Chunk::next.
  • Iterator that iterates over previous chunks by continuously calling Chunk::prev.
  • Provides statistics about the memory usage of the bump allocator.