bump_scope::stats

Struct Chunk

pub struct Chunk<'a> { /* private fields */ }
Refers to a chunk of memory that was allocated by the bump allocator.

See Stats.

Implementations

Source

impl<'a> Chunk<'a>

pub fn prev(self) -> Option<Self>

Returns the previous (smaller) chunk.

pub fn next(self) -> Option<Self>

Returns the next (bigger) chunk.

pub fn iter_prev(self) -> ChunkPrevIter<'a>

Returns an iterator over all previous (smaller) chunks.

pub fn iter_next(self) -> ChunkNextIter<'a>

Returns an iterator over all next (bigger) chunks.

pub fn size(self) -> usize

Returns the size of this chunk in bytes.

pub fn capacity(self) -> usize

Returns the capacity of this chunk in bytes.

pub fn allocated(self) -> usize

Returns the amount of allocated bytes. This includes padding and wasted space due to reallocations.

pub fn remaining(self) -> usize

Returns the remaining capacity.

pub fn chunk_start(self) -> NonNull<u8>

Returns a pointer to the start of the chunk.

pub fn chunk_end(self) -> NonNull<u8>

Returns a pointer to the end of the chunk.

pub fn content_start(self) -> NonNull<u8>

Returns a pointer to the start of the chunk’s content.

pub fn content_end(self) -> NonNull<u8>

Returns a pointer to the end of the chunk’s content.

pub fn bump_position(self) -> NonNull<u8>

Returns the bump pointer. It lies within the chunk’s content range.