bump_scope

Struct Stats

pub struct Stats<'a, const GUARANTEED_ALLOCATED: bool = false> { /* private fields */ }
Provides statistics about the memory usage of the bump allocator.

This is returned from the stats method of Bump, BumpScope, BumpScopeGuard, BumpVec, …

Implementations

Source

impl<'a, const GUARANTEED_ALLOCATED: bool> Stats<'a, GUARANTEED_ALLOCATED>

pub fn count(self) -> usize

Returns the number of chunks.

pub fn size(self) -> usize

Returns the total size of all chunks.

pub fn capacity(self) -> usize

Returns the total capacity of all chunks.

pub fn allocated(self) -> usize

Returns the amount of allocated bytes. This includes padding and wasted space due to reallocations.

pub fn remaining(self) -> usize

Returns the total remaining capacity of all chunks.

pub fn small_to_big(self) -> ChunkNextIter<'a>

Returns an iterator from smallest to biggest chunk.

pub fn big_to_small(self) -> ChunkPrevIter<'a>

Returns an iterator from biggest to smallest chunk.

pub fn guaranteed_allocated(self) -> Option<Stats<'a, true>>

Turns this Stats into a Stats where GUARANTEED_ALLOCATED = true.

pub fn not_guaranteed_allocated(self) -> Stats<'a, false>

Turns this Stats into a Stats where GUARANTEED_ALLOCATED = false.

Source

impl<'a> Stats<'a, true>

pub fn current_chunk(self) -> Chunk<'a>

This is the chunk we are currently allocating on.

Source

impl<'a> Stats<'a, false>

pub fn current_chunk(self) -> Option<Chunk<'a>>

This is the chunk we are currently allocating on.