Struct bump_scope::UninitStats
source · pub struct UninitStats<'a, const UP: bool> {
pub current: Option<Chunk<'a, UP>>,
}Expand description
Provides statistics about the memory usage of the bump allocator.
This is returned from the stats method of Bump, BumpScope, BumpScopeGuard, BumpVec, …
Fields
current: Option<Chunk<'a, UP>>This is the chunk we are currently allocating on.
Implementations
sourceimpl<'a, const UP: bool> UninitStats<'a, UP>
impl<'a, const UP: bool> UninitStats<'a, UP>
sourcepub fn allocated(self) -> usize
pub fn allocated(self) -> usize
Returns the amount of allocated bytes. This includes padding and wasted space due to reallocations.
sourcepub fn small_to_big(self) -> ChunkNextIter<'a, UP> ⓘ
pub fn small_to_big(self) -> ChunkNextIter<'a, UP> ⓘ
Returns an iterator from smallest to biggest chunk.
sourcepub fn big_to_small(self) -> ChunkPrevIter<'a, UP> ⓘ
pub fn big_to_small(self) -> ChunkPrevIter<'a, UP> ⓘ
Returns an iterator from biggest to smallest chunk.