Struct bump_scope::WithoutShrink
source · pub struct WithoutShrink<A>(pub A);Expand description
Wraps an bump allocator and does nothing on shrink.
This type only implements Allocator for wrapped types that implement BumpAllocator, so you don’t accidentally leak memory.
Tuple Fields
0: AImplementations
sourceimpl<A> WithoutShrink<A>
impl<A> WithoutShrink<A>
sourcepub fn without_dealloc(self) -> WithoutDealloc<Self>
pub fn without_dealloc(self) -> WithoutDealloc<Self>
Wraps self in WithoutDealloc so that deallocate becomes a no-op.