Trait bump_scope::NoDrop

source ·
pub trait NoDrop { }
This trait marks types that don’t need dropping.

This trait is a best effort for modeling such a constraint. It is not implemented for all types that don’t need dropping.

Every T where T: Copy and every [T] where T: NoDrop automatically implements NoDrop.

It is used as a bound for BumpBox’s into_ref and into_mut so you don’t accidentally omit a drop that does matter.

Implementations on Foreign Types

source

impl NoDrop for str

source

impl<T: NoDrop> NoDrop for [T]

Implementors

source

impl NoDrop for FixedBumpString<'_>

source

impl<A, const MIN_ALIGN: usize, const UP: bool> NoDrop for BumpScope<'_, A, MIN_ALIGN, UP>

source

impl<T> NoDrop for BumpBox<'_, T>
where T: NoDrop,

source

impl<T> NoDrop for FixedBumpVec<'_, T>
where T: NoDrop,

source

impl<T: Copy> NoDrop for T