Trait bump_scope::NoDrop
source · pub trait NoDrop { }Expand description
This trait marks types that don’t need dropping.
This trait is a best effort for modeling such a constraint. It is not implemented for all types that don’t need dropping.
Every T where T: Copy and every [T] where T: NoDrop automatically implements NoDrop.
It is used as a bound for BumpBox’s into_ref and into_mut so you don’t accidentally omit a drop that does matter.